﻿using System.IO;
using System.Text;

namespace DocAssistShared
{
    /// <summary>
    ///  A helper class that provides some common file manipulation functionalitis
    /// </summary>
    public static class FileHelper
    {
        #region Enumeration
        
        /// <summary>
        ///  An enumeration whose values indicate if a path is a file or directory or invalid
        /// </summary>
        public enum FileType
        {
            File,
            Directory,
            NonExistent
        }

        #endregion

        #region Methods

        /// <summary>
        ///  returns the file type of the specified path
        /// </summary>
        /// <param name="path">The path to return the file type of</param>
        /// <returns>The file type of the specified path</returns>
        public static FileType GetPathFileType(this string path)
        {
            if (File.Exists(path)) return FileType.File;

            if (Directory.Exists(path)) return FileType.Directory;

            return FileType.NonExistent;
        }

        /// <summary>
        ///  adds a back-slash to the specified string if it doesn't exist
        /// </summary>
        /// <param name="path">The path string a back-slash is to be added to</param>
        /// <returns>The path string with a back-slash</returns>
        public static string AddBackSlashForFolderIfNeeded(this string path)
        {
            return path.LastIndexOf('\\') == path.Length
                       ? path
                       : new StringBuilder(path).Append('\\').ToString();
        }

        /// <summary>
        ///  returns a distinct path as per the type of the path by adding a tailing back-slash to
        ///  the specified path string if it represents a folder (directory) or leaving it as-is
        /// </summary>
        /// <param name="path">The path to be differentiated</param>
        /// <returns>The distinct path for the specified path</returns>
        public static string GetDistinctPath(this string path)
        {
            var ft = GetPathFileType(path);
            if (ft == FileType.Directory)
                path = AddBackSlashForFolderIfNeeded(path);

            return path;
        }

        #endregion
    }
}
